#
#   ic.py
#
#   Python script to create IC footprints
#
#   (c)2012 Brian Sidebotham - www.valvers.com
#
#   IC information was taken from the Texas Instruments outline pages:
#
#   See:
#   http://www.ti.com/sc/docs/psheets/type/type.html
#
#

import kicad as KiCad
from kicad import mm as mm

def qfp(pins_in, bodysize, pitch):
    """
    Pass a simple int to pins if all sides are equal. If sides are not equal,
    pass a tuple of length two for pins listing the number of pins on each
    side of the device.

    Bodysize should be a tuple of length 2, in the form (x, y)

    The pitch is a float giving the pitch of the pins in mm
    """

    # Test to see if all sides are the same size, or if there are different
    # pin counts for the x and y sides
    if isinstance(pins_in, tuple):
        pins = pins_in
    else:
        pins = (pins_in/4, pins_in/4)

    # Work out the total package pincount from the four sides (2x and 2y)
    pincount = (pins[0]*2) + (pins[1]*2)

    # Insantiate a module and set it's name according to the current naming
    # convention
    result = KiCad.Module()
    result.fields[0].text = "QFP{0}-{1}x{2}-{3}".format(
            str(pincount),
            str(bodysize[0]),
            str(bodysize[1]),
            str(pitch) )
    result.fields[0].y = -mm(2)
    result.fields[0].visible = KiCad.Const.FIELDVIS_INVISIBLE


    # A line width (in mm) which can be used to draw on the silk screen layer
    linewidth = 0.2

    # Mark the body outline (centred at 0,0)
    result.addDrawing(
        KiCad.DrawRectangle(
            size = (mm(bodysize[0]), mm(bodysize[1])),
            lwidth = mm(linewidth),
            layer = KiCad.Const.LAYER_TOP_SILK ) )

    # Setup the pad sizes
    padsizex = mm(pitch - 0.2)
    padsizey = mm(1)

    # Limit the width of a pad...
    if padsizex > mm(0.6):
        padsizex = mm(0.6)

    # Add pins for the bottom row (starting at pin 1)
    pin_number = 1

    # Side one
    startx = 0 - mm(((pins[0] - 1) * pitch) / 2)
    starty = mm((bodysize[1] / 2) + 0.75)

    while pin_number <= (pins[0]):

        p = KiCad.Pad( position = (startx + mm( (pin_number - 1) * pitch), starty),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Mark Pin 1 with a rectangle (would be better to be small arrow head)
    result.addDrawing(
            KiCad.DrawRectangle(
                    centre = (startx, mm((bodysize[1]/2) - 0.35)),
                    size = (mm(0.2), mm(0.2)),
                    lwidth = mm(linewidth),
                    layer = KiCad.Const.LAYER_TOP_SILK ) )

    # Side two
    startx = mm((bodysize[0] / 2) + 0.75)
    starty = mm(((pins[1]-1) * pitch) / 2)

    while pin_number <= (pins[0] + pins[1]):

        p = KiCad.Pad( position=(startx, starty - mm( ((pin_number - pins[0]) - 1) * pitch)),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_90DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Side three
    startx = mm(((pins[0]-1) * pitch) / 2)
    starty = 0 - mm((bodysize[1] / 2) + 0.75)

    while pin_number <= (pins[0] + pins[1] + pins[0]):

        p = KiCad.Pad( position=(startx - mm( ((pin_number - (pins[0] + pins[1])) - 1) * pitch), starty),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Side four
    startx = 0 - mm((bodysize[0] / 2) + 0.75)
    starty = 0 - mm(((pins[1]-1) * pitch) / 2)

    while pin_number <= (pins[0] + pins[1] + pins[0] + pins[1]):

        p = KiCad.Pad( position=(startx, starty + mm( ((pin_number - (pins[0] + pins[1] + pins[0])) - 1) * pitch)),
                 size=(padsizex, padsizey),
                 number=pin_number,
                orientation = KiCad.Const.ORIENTATION_90DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype= KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    return result

#
#
#
#
#
#
#
#
#
#
#
#
#


def plcc(pins_in, bodysize, pitch):
    """
    Pass a simple int to pins if all sides are equal. If sides are not equal,
    pass a tuple of length two for pins listing the number of pins on each
    side of the device.

    Bodysize should be a tuple of length 2, in the form (x, y)

    The pitch is a float giving the pitch of the pins in mm
    """

    # Test to see if all sides are the same size, or if there are different
    # pin counts for the x and y sides
    if isinstance(pins_in, tuple):
        pins = pins_in
    else:
        pins = (pins_in/4, pins_in/4)

    # Work out the total package pincount from the four sides (2x and 2y)
    pincount = (pins[0]*2) + (pins[1]*2)

    # Insantiate a module and set it's name according to the current naming
    # convention
    result = KiCad.Module()
    result.fields[0].text = "PLCC{0}-{1}x{2}-{3}".format(
            str(pincount),
            str(bodysize[0]),
            str(bodysize[1]),
            str(pitch) )
    result.fields[0].y = -mm(2)
    result.fields[0].visible = KiCad.Const.FIELDVIS_INVISIBLE

    # A line width (in mm) which can be used to draw on the silk screen layer
    linewidth = 0.2

    # Mark the body outline (centred at 0,0)
    result.addDrawing(
        KiCad.DrawRectangle(
            size = (mm(bodysize[0]), mm(bodysize[1])),
            lwidth = mm(linewidth),
            layer = KiCad.Const.LAYER_TOP_SILK ) )

    # Setup the pad sizes
    padsizex = mm(pitch - 0.2)
    padsizey = mm(1.5)

    # Pin offsets are constant, (bodysize - 2.62) / 2

    # Limit the width of a pad...
    if padsizex > mm(0.6):
        padsizex = mm(0.6)

    # Add pins for the bottom row (starting at pin 1)
    pin_number = 1

    # Side one, pt.1 Pin 1 is in the middle, so at x = 0
    startx = 0
    starty = 0 - mm(((bodysize[1] / 2) - (2.62 / 2)) + 1)

    # We are only doing part of the
    while pin_number <= ((pins[0] + 1) / 2):

        p = KiCad.Pad( position = (startx - mm( (pin_number - 1) * pitch), starty),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Mark Pin 1 with a rectangle (would be better to be small arrow head)
    result.addDrawing(
            KiCad.DrawRectangle(
                    centre = (startx, 0 - mm((bodysize[1]/2) - 0.35)),
                    size = (mm(0.2), mm(0.2)),
                    lwidth = mm(linewidth),
                    layer = KiCad.Const.LAYER_TOP_SILK ) )

    # Side two
    startx = 0 - mm(((bodysize[0] / 2) - (2.62 / 2)) + 1)
    starty = 0 - mm(((pins[1]-1) * pitch) / 2)

    while pin_number <= (((pins[0] + 1) / 2) + pins[1]):

        p = KiCad.Pad( position=(startx, starty + mm( ((pin_number - ((pins[0] + 1) / 2)) - 1) * pitch)),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_90DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Side three
    startx = 0 - mm(((pins[0]-1) * pitch) / 2)
    starty = mm(((bodysize[0] / 2) - (2.62 / 2)) + 1)

    while pin_number <= (((pins[0] + 1) / 2) + pins[1] + pins[0]):

        p = KiCad.Pad( position=(startx + mm( ((pin_number - (((pins[0] + 1) / 2) + pins[1])) - 1) * pitch), starty),
                size = (padsizex, padsizey),
                number = pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Side four
    startx = mm(((bodysize[0] / 2) - (2.62 / 2)) + 1)
    starty = mm(((pins[1]-1) * pitch) / 2)

    while pin_number <= (((pins[0] + 1) / 2) + pins[1] + pins[0] + pins[1]):

        p = KiCad.Pad( position=(startx, starty - mm( ((pin_number - (((pins[0] + 1) / 2) + pins[1] + pins[0])) - 1) * pitch)),
                size=(padsizex, padsizey),
                number=pin_number,
                orientation = KiCad.Const.ORIENTATION_90DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype= KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Side one pt.2
    startx = mm(((pins[0]-1) * pitch) / 2)
    starty = 0 - mm((bodysize[1] / 2) - (2.62 / 2) + 1)

    while pin_number <= (pins[0] + pins[1] + pins[0] + pins[1]):

        p = KiCad.Pad( position=(startx - mm( ((pin_number - (((pins[0] + 1) / 2) + pins[1] + pins[0] + pins[1])) - 1) * pitch), starty),
                size=(padsizex, padsizey),
                number=pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype= KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1


    return result














def soic(pins, bodysize, pitch):
    """
    @param bodysize is a tuple!
    """
    result = KiCad.Module()
    result.fields[0].text = "SOIC{0}-{1}x{2}-{3}".format(str(pins), str(bodysize[0]), str(bodysize[1]), str(pitch))
    result.fields[0].y = -mm(2)
    result.fields[0].visible = KiCad.Const.FIELDVIS_INVISIBLE

    # A line width (in mm)
    linewidth = 0.2

    # Mark the body outline (centred at 0,0)
    result.addDrawing(
        KiCad.DrawRectangle(
            size=(mm(bodysize[0]), mm(bodysize[1])),
            lwidth=mm(linewidth),
            layer=KiCad.Const.LAYER_TOP_SILK ) )

    # Generate a pad size suitable for this device. Limit the maximum pad size
    # vertically
    padsizex = mm(1)
    padsizey = mm(pitch - 0.2)

    if padsizey > 0.65:
        padsizey = 0.65

    pin_number = 1

    # Side one
    starty = 0 - mm((((pins / 2)-1) * pitch) / 2)
    startx = 0 - mm((bodysize[0] / 2) + 1)

    while pin_number <= ((pins / 2) * 1):

        p = KiCad.Pad( position=(startx, starty + mm( (pin_number - 1) * pitch)),
                 size=(padsizex, padsizey),
                 number=pin_number,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype= KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    # Mark Pin 1 with a rectangle (would be better to be small arrow head)
    result.addDrawing(
        KiCad.DrawRectangle(
            centre=(0 - mm((bodysize[0]/2) - 0.35), starty),
            size=(mm(0.2), mm(0.2)),
            lwidth=mm(linewidth),
            layer=KiCad.Const.LAYER_TOP_SILK ) )

    # Side two
    startx = mm((bodysize[0] / 2) + 1)
    starty = mm((((pins / 2)-1) * pitch) / 2)

    while pin_number <= ((pins / 2) * 2):

        p = KiCad.Pad( position=(startx, starty - mm( ((pin_number - ((pins/2)*1)) - 1) * pitch)),
                 size=(padsizex, padsizey),
                 number=pin_number,
                orientation = KiCad.Const.ORIENTATION_90DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype= KiCad.Const.PADTYPE_SMD )

        result.addPad(p)
        pin_number += 1

    return result


